@@ -1,7 +1,6 @@
 """Drive service."""
 
 import io
-import json
 import logging
 import mimetypes
 import os
@@ -48,30 +47,33 @@ class DriveService(BaseService):
             if cookie.name == COOKIE_APPLE_WEBAUTH_VALIDATE and cookie.value:
                 match: Optional[Match[str]] = search(r"\bt=([^:]+)", cookie.value)
                 if match is None:
-                    raise TokenException("Can't extract token from %r" % cookie.value)
+                    raise TokenException(f"Can't extract token from {cookie.value}")
                 return {"token": match.group(1)}
         raise TokenException("Token cookie not found")
 
-    def get_node_data(self, drivewsid):
+    def get_node_data(
+        self, drivewsid: str, share_id: Optional[dict[str, Any]] = None
+    ) -> dict[str, Any]:
         """Returns the node data."""
+        payload = [
+            {
+                "drivewsid": drivewsid,
+                "partialData": False,
+            }
+        ]
+        if share_id:
+            payload[0]["shareID"] = share_id
         request: Response = self.session.post(
             self.service_root + "/retrieveItemDetailsInFolders",
             params=self.params,
-            data=json.dumps(
-                [
-                    {
-                        "drivewsid": drivewsid,
-                        "partialData": False,
-                    }
-                ]
-            ),
+            json=payload,
         )
         self._raise_if_error(request)
         return request.json()[0]
 
     def get_file(self, file_id: str, zone: str = CLOUD_DOCS_ZONE, **kwargs) -> Response:
         """Returns iCloud Drive file."""
-        file_params = dict(self.params)
+        file_params: dict[str, Any] = dict(self.params)
         file_params.update({"document_id": file_id})
         response: Response = self.session.get(
             self._document_root + f"/ws/{zone}/download/by_id",
@@ -90,7 +92,8 @@ class DriveService(BaseService):
     def get_app_data(self):
         """Returns the app library (previously ubiquity)."""
         request: Response = self.session.get(
-            self.service_root + "/retrieveAppLibraries", params=self.params
+            self.service_root + "/retrieveAppLibraries",
+            params=self.params,
         )
         self._raise_if_error(request)
         return request.json()["items"]
@@ -119,17 +122,15 @@ class DriveService(BaseService):
             self._document_root + f"/ws/{zone}/upload/web",
             params=file_params,
             headers={CONTENT_TYPE: CONTENT_TYPE_TEXT},
-            data=json.dumps(
-                {
-                    "filename": file_object.name,
-                    "type": "FILE",
-                    "content_type": content_type,
-                    "size": file_size,
-                }
-            ),
+            json={
+                "filename": file_object.name,
+                "type": "FILE",
+                "content_type": content_type,
+                "size": file_size,
+            },
         )
         self._raise_if_error(request)
-        return (request.json()[0]["document_id"], request.json()[0]["url"])
+        return request.json()[0]["document_id"], request.json()[0]["url"]
 
     def _update_contentws(
         self,
@@ -172,7 +173,7 @@ class DriveService(BaseService):
             self._document_root + f"/ws/{zone}/update/documents",
             params=self.params,
             headers={CONTENT_TYPE: CONTENT_TYPE_TEXT},
-            data=json.dumps(data),
+            json=data,
         )
         self._raise_if_error(request)
         return request.json()
@@ -211,17 +212,15 @@ class DriveService(BaseService):
             self.service_root + "/createFolders",
             params=self.params,
             headers={CONTENT_TYPE: CONTENT_TYPE_TEXT},
-            data=json.dumps(
-                {
-                    "destinationDrivewsId": parent,
-                    "folders": [
-                        {
-                            "clientId": temp_client_id,
-                            "name": name,
-                        }
-                    ],
-                }
-            ),
+            json={
+                "destinationDrivewsId": parent,
+                "folders": [
+                    {
+                        "clientId": temp_client_id,
+                        "name": name,
+                    }
+                ],
+            },
         )
         self._raise_if_error(request)
         return request.json()
@@ -231,17 +230,15 @@ class DriveService(BaseService):
         request: Response = self.session.post(
             self.service_root + "/deleteItems",
             params=self.params,
-            data=json.dumps(
-                {
-                    "items": [
-                        {
-                            "drivewsid": node_id,
-                            "etag": etag,
-                            "clientId": self.params["clientId"],
-                        }
-                    ],
-                }
-            ),
+            json={
+                "items": [
+                    {
+                        "drivewsid": node_id,
+                        "etag": etag,
+                        "clientId": self.params["clientId"],
+                    }
+                ],
+            },
         )
         self._raise_if_error(request)
         return request.json()
@@ -251,17 +248,15 @@ class DriveService(BaseService):
         request: Response = self.session.post(
             self.service_root + "/renameItems",
             params=self.params,
-            data=json.dumps(
-                {
-                    "items": [
-                        {
-                            "drivewsid": node_id,
-                            "etag": etag,
-                            "name": name,
-                        }
-                    ],
-                }
-            ),
+            json={
+                "items": [
+                    {
+                        "drivewsid": node_id,
+                        "etag": etag,
+                        "name": name,
+                    }
+                ],
+            },
         )
         self._raise_if_error(request)
         return request.json()
@@ -273,17 +268,15 @@ class DriveService(BaseService):
         request: Response = self.session.post(
             self.service_root + "/moveItemsToTrash",
             params=self.params,
-            data=json.dumps(
-                {
-                    "items": [
-                        {
-                            "drivewsid": node_id,
-                            "etag": etag,
-                            "clientId": temp_client_id,
-                        }
-                    ],
-                }
-            ),
+            json={
+                "items": [
+                    {
+                        "drivewsid": node_id,
+                        "etag": etag,
+                        "clientId": temp_client_id,
+                    }
+                ],
+            },
         )
         self._raise_if_error(request)
         return request.json()
@@ -293,16 +286,14 @@ class DriveService(BaseService):
         request: Response = self.session.post(
             self.service_root + "/putBackItemsFromTrash",
             params=self.params,
-            data=json.dumps(
-                {
-                    "items": [
-                        {
-                            "drivewsid": node_id,
-                            "etag": etag,
-                        }
-                    ],
-                }
-            ),
+            json={
+                "items": [
+                    {
+                        "drivewsid": node_id,
+                        "etag": etag,
+                    }
+                ],
+            },
         )
         self._raise_if_error(request)
         return request.json()
@@ -312,16 +303,14 @@ class DriveService(BaseService):
         request: Response = self.session.post(
             self.service_root + "/deleteItems",
             params=self.params,
-            data=json.dumps(
-                {
-                    "items": [
-                        {
-                            "drivewsid": node_id,
-                            "etag": etag,
-                        }
-                    ],
-                }
-            ),
+            json={
+                "items": [
+                    {
+                        "drivewsid": node_id,
+                        "etag": etag,
+                    }
+                ],
+            },
         )
         self._raise_if_error(request)
         return request.json()
@@ -376,9 +365,9 @@ class DriveNode:
     NAME_ROOT = "root"
     NAME_UNKNOWN = "<UNKNOWN>"
 
-    def __init__(self, conn, data) -> None:
-        self.data = data
-        self.connection = conn
+    def __init__(self, conn: DriveService, data: dict[str, Any]) -> None:
+        self.data: dict[str, Any] = data
+        self.connection: DriveService = conn
         self._children: Optional[list[DriveNode]] = None
 
     @property
@@ -417,9 +406,13 @@ class DriveNode:
         """Gets the node children."""
         if not self._children or force:
             if "items" not in self.data or force:
-                self.data.update(self.connection.get_node_data(self.data["drivewsid"]))
+                self.data.update(
+                    self.connection.get_node_data(
+                        self.data["drivewsid"], self.data.get("shareID")
+                    )
+                )
             if "items" not in self.data:
-                raise KeyError("No items in folder, status: %s" % self.data["status"])
+                raise KeyError(f"No items in folder, status: {self.data['status']}")
             self._children = [
                 DriveNode(self.connection, item_data)
                 for item_data in self.data["items"]
@@ -510,8 +503,7 @@ class DriveNode:
             return self.connection.recover_items_from_trash(
                 self.data["drivewsid"], self.data["etag"]
             )
-        else:
-            raise ValueError(f"'{self.name}' does not appear to be in the Trash.")
+        raise ValueError(f"'{self.name}' does not appear to be in the Trash.")
 
     def delete_forever(self):
         """Permanently deletes an iCloud Drive item from trash."""
@@ -520,11 +512,10 @@ class DriveNode:
             return self.connection.delete_forever_from_trash(
                 self.data["drivewsid"], self.data["etag"]
             )
-        else:
-            raise ValueError(
-                f"'{self.name}' does not appear to be in the Trash. Please 'delete()' it first before "
-                f"trying to 'delete_forever()'."
-            )
+        raise ValueError(
+            f"'{self.name}' does not appear to be in the Trash. Please 'delete()' it first before "
+            f"trying to 'delete_forever()'."
+        )
 
     def get(self, name: str) -> "DriveNode":
         """Gets the node child."""
